const fs = require('fs');

var express = require('express');
var router = express.Router();

const Datastore = require('nedb-promise'),
      db = new Datastore({ filename: '/home/tjhintz/tideable-db', autoload: true });

const { processSources } = require('./sourceData.js');
const sources = JSON.parse(fs.readFileSync('../sources.json')).sources
      .sort((a, b) => parseInt(b.marine.lat) - parseInt(a.marine.lat));

let data = {};
sources.forEach(source => {
    const id = source.id;
    data[id] = {
        // tides: JSON.parse(fs.readFileSync(`./routes/data/${id}-tide.json`)).data,
        // waves: JSON.parse(fs.readFileSync(`./routes/data/${id}-marine-forecast.json`)).data,
        // weather: JSON.parse(fs.readFileSync(`./routes/data/${id}-weather.json`)).data,
        name: source.name,
        urls: source.urls
    };
});

function addDays(date, days) {
    var result = new Date(date);
    result.setDate(result.getDate() + days);
    return result;
}

router.get('/location/:location/date/:date', function(req, res, next) {
    let date = new Date(req.params.date),
        toDate = addDays(date, 1),
        dateMonth = new Date(date.getFullYear(), 1, 1),
        target = data[req.params.location];
    Promise.all([db.cfind({ date: { $gte: toDate },
                            distance: { $lt: -0.3 },
                            location: req.params.location, type: 'tide' })
                 .sort({ date: 1 })
                 .limit(1)
                 .exec(),
                 db
                 .cfind({ $and: [{ date: { $gte: date }},
                                 { date: { $lt: toDate }}],
                          location: req.params.location })
                 .exec(),
                 db
                 .cfind({ $and: [{ date: { $gte: dateMonth }},
                                 { date: { $lte: addDays(dateMonth, 365) }}],
                          distance: { $lt: -0.3 },
                          location: req.params.location, type: 'tide' })
                 .exec()])
        .then(([nextTideable, items, tideableDays]) =>
              res.json({ tides: items.filter(x => x.type === 'tide' && x.low)
                         .sort((a, b) => a.date - b.date),
                         waves: items.filter(x => x.type === 'marine'),
                         weather: items.filter(x => x.type === 'weather')
                         .sort((a, b) => a.date - b.date)[0],
                         name: target.name,
                         nextTideable: nextTideable[0],
                         urls: target.urls,
                         tideableDays: tideableDays.map(x => x.date)
                         .filter(x => x.getHours() < 22 && x.getHours() > 5)
                       }));
});

router.get('/location/:location/tide/:date', function(req, res, next) {
    let date = addDays(new Date(req.params.date), -64),
        toDate = addDays(date, 365),
        target = data[req.params.location];
    Promise.all([db
                 .cfind({ $and: [{ date: { $gte: date }},
                                 { date: { $lte: toDate }}],
                          distance: { $lt: -0.3 },
                          location: req.params.location, type: 'tide' })
                 .exec(),
                 db
                 .cfind({ $and: [{ date: { $gte: date }},
                                 { date: { $lte: toDate }}],
                          $and: [{ distance: { $lte: 0 }},
                                 { distance: { $gte: -0.3 }}],
                          location: req.params.location, type: 'tide' })
                 .exec()])
        .then(([goodTides, okTides]) =>
              res.json({ goodTides: goodTides.map(x => x.date),
                         okTides: okTides.map(x => x.date)
                       }));
});

router.post('/refresh-sources', function(req, res, next) {
    Promise.all(processSources(db)).then(() => res.json({}));
})

router.post('/clear-tide', (req, res, next) =>
            db.remove({ type: 'tide' }, { multi: true }).then(() => res.json({})));

module.exports = router;

// const moment = require('moment');
// const doc = { hello: 'world'
//                , n: 5
//               , date: moment("2020/03/02" + ' ' + "11:03 AM", 'YYYY-MM-DD LT').toDate()
//                , nedbIsAwesome: true
//                , notthere: null
//                , notToBeSaved: undefined  // Will not be saved
//                , fruits: [ 'apple', 'orange', 'pear' ]
//                , infos: { name: 'nedb' }
//                };

// let date = new Date('2020-03-15T00:00:00.000Z'),
//     toDate = addDays(date, 1);
// db
//     .cfind({ date: { $gt: date },
//              distance: { $lt: -0.3 },
//              location: 'asilomar', type: 'tide' })
//     .sort({ date: 1 })
//     .limit(1)
//     .exec()
//     .then(x => console.log(x));

// db.insert(doc)
//     .then(db.findOne({ date: { $lt: new Date('2020-03-04T08:00:00.000Z') }}))
//     .then(x => console.log(x.date.toLocaleTimeString()))
//     .then(db.remove({}));

// db.cfind({ $and: [{ date: { $lt: new Date('2020-03-05T00:00:00.000Z') }},
//                  { date: { $gt: new Date('2020-03-04T00:00:00.000Z') }}] })
//     .sort({ location: -1, date: -1 })
//     .exec()
//     .then(x => console.log(x));
// .then(db.remove({}, { multi: true }));

// db.cfind({ type: 'tide', location: 'asilomar', date: new Date('2020-03-04T06:43:00.000Z') })
//     .exec()
//     .then(x => console.log(x));

// db.remove({ _id: 'BJAkPl8Y9G4rn6GM' }, {})
//     .then(db.findOne({ $and: [{ date: { $lt: new Date('2020-03-04T00:00:00.000Z') }},
//                               { date: { $gt: new Date('2020-03-03T00:00:00.000Z') }}] }))
//     .then(x => console.log(x));

// db.findOne({ n: { $lt: 4 }})
//     .then(x => console.log(x));
